@@ -1,4 +1,4 @@
-__version__ = "6.4.0"
+__version__ = "6.19.1"
 
 from smartcar.auth_client import AuthClient
 
@@ -12,6 +12,8 @@ from smartcar.smartcar import (
     get_api_version,
     set_api_version,
     verify_payload,
+    get_connections,
+    delete_connections,
 )
 
 from smartcar.vehicle import Vehicle
