@@ -1,10 +1,11 @@
 from collections import namedtuple
 import json
-from typing import List
+from typing import Callable, List, Optional
 import smartcar.config as config
 import smartcar.helpers as helpers
 import smartcar.smartcar
-import smartcar.types as types
+import smartcar.v2_response as v2
+import smartcar.v3_response as v3
 import smartcar.exception as sce
 
 
@@ -50,7 +51,49 @@ class Vehicle(object):
             if options.get("flags"):
                 self._flags = options["flags"]
 
-    def vin(self) -> types.Vin:
+    def get_signals(self) -> v3.Response:
+        """
+        GET Vehicle.signals
+
+        Returns:
+            Response: smartcar.response.v3.Response
+        """
+        origin = config.VEHICLE_API_ORIGIN
+        api_version = "3"
+        path = "signals"
+        url = self._format_url(path, origin=origin, api_version=api_version)
+        headers = self._get_headers()
+        response = helpers.requester("GET", url, headers=headers)
+
+        return {
+            "body": response.json(),
+            "headers": response.headers,
+        }
+
+    def get_signal(self, signal_code: str) -> v3.Response:
+        """
+        GET Vehicle.signals/{signal_code}
+
+        Args:
+            signal_code (str): The code of the signal to retrieve.
+            e.g. "odometer-traveleddistance"
+
+        Returns:
+            Response: smartcar.response.v3.Response
+        """
+        origin = config.VEHICLE_API_ORIGIN
+        api_version = "3"
+        path = f"signals/{signal_code}"
+        url = self._format_url(path, origin=origin, api_version=api_version)
+        headers = self._get_headers()
+        response = helpers.requester("GET", url, headers=headers)
+
+        return {
+            "body": response.json(),
+            "headers": response.headers,
+        }
+
+    def vin(self) -> v2.Vin:
         """
         GET Vehicle.vin
 
@@ -64,9 +107,9 @@ class Vehicle(object):
         url = self._format_url(path)
         headers = self._get_headers()
         response = helpers.requester("GET", url, headers=headers)
-        return types.select_named_tuple(path, response)
+        return v2.select_named_tuple(path, response)
 
-    def charge(self) -> types.Charge:
+    def charge(self) -> v2.Charge:
         """
         GET Vehicle.charge
 
@@ -80,9 +123,9 @@ class Vehicle(object):
         url = self._format_url(path)
         headers = self._get_headers()
         response = helpers.requester("GET", url, headers=headers)
-        return types.select_named_tuple(path, response)
+        return v2.select_named_tuple(path, response)
 
-    def battery(self) -> types.Battery:
+    def battery(self) -> v2.Battery:
         """
         GET Vehicle.battery
 
@@ -98,9 +141,9 @@ class Vehicle(object):
         url = self._format_url(path)
         headers = self._get_headers()
         response = helpers.requester("GET", url, headers=headers)
-        return types.select_named_tuple(path, response)
+        return v2.select_named_tuple(path, response)
 
-    def battery_capacity(self) -> types.BatteryCapacity:
+    def battery_capacity(self) -> v2.BatteryCapacity:
         """
         GET Vehicle.battery_capacity
 
@@ -114,9 +157,31 @@ class Vehicle(object):
         url = self._format_url(path)
         headers = self._get_headers()
         response = helpers.requester("GET", url, headers=headers)
-        return types.select_named_tuple(path, response)
+        return v2.select_named_tuple(path, response)
 
-    def fuel(self) -> types.Fuel:
+    def nominal_capacity(self) -> v2.NominalCapcity:
+        """
+        GET Vehicle.nominal_capacity
+        Returns:
+            NamedTuple(
+                "NominalCapcity",
+                [
+                    ("availableCapacities", List[AvailableCapacity]),
+                    ("capacity", Optional[SelectedCapacity]),
+                    ("url", Optional[str]),
+                    ("meta", namedtuple),
+                ],
+            )
+        Raises:
+            SmartcarException
+        """
+        path = "battery/nominal_capacity"
+        url = self._format_url(path)
+        headers = self._get_headers()
+        response = helpers.requester("GET", url, headers=headers)
+        return v2.select_named_tuple(path, response)
+
+    def fuel(self) -> v2.Fuel:
         """
         GET Vehicle.fuel
 
@@ -131,9 +196,9 @@ class Vehicle(object):
         url = self._format_url(path)
         headers = self._get_headers()
         response = helpers.requester("GET", url, headers=headers)
-        return types.select_named_tuple(path, response)
+        return v2.select_named_tuple(path, response)
 
-    def tire_pressure(self) -> types.TirePressure:
+    def tire_pressure(self) -> v2.TirePressure:
         """
         GET Vehicle.tire_pressure
 
@@ -150,9 +215,9 @@ class Vehicle(object):
         url = self._format_url(path)
         headers = self._get_headers()
         response = helpers.requester("GET", url, headers=headers)
-        return types.select_named_tuple(path, response)
+        return v2.select_named_tuple(path, response)
 
-    def engine_oil(self) -> types.EngineOil:
+    def engine_oil(self) -> v2.EngineOil:
         """
         GET Vehicle.engine_oil
 
@@ -166,9 +231,9 @@ class Vehicle(object):
         url = self._format_url(path)
         headers = self._get_headers()
         response = helpers.requester("GET", url, headers=headers)
-        return types.select_named_tuple(path, response)
+        return v2.select_named_tuple(path, response)
 
-    def odometer(self) -> types.Odometer:
+    def odometer(self) -> v2.Odometer:
         """
         GET Vehicle.odometer
 
@@ -182,9 +247,82 @@ class Vehicle(object):
         url = self._format_url(path)
         headers = self._get_headers()
         response = helpers.requester("GET", url, headers=headers)
-        return types.select_named_tuple(path, response)
+        return v2.select_named_tuple(path, response)
 
-    def location(self) -> types.Location:
+    def service_history(
+        self, start_date: Optional[str] = None, end_date: Optional[str] = None
+    ) -> v2.ServiceHistory:
+        """
+        Returns a list of all the service records performed on the vehicle,
+        filtered by the optional date range. If no dates are specified, records from the
+        last year are returned.
+
+        Args:
+            start_date (Optional[str]): The start date for the record filter, either in 'YYYY-MM-DD' or
+                                        'YYYY-MM-DDTHH:MM:SS.SSSZ' format.
+            end_date (Optional[str]): The end date for the record filter, similar format to start_date.
+
+        Returns:
+            ServiceHistory: NamedTuple("ServiceHistory", [("items", List[ServiceRecord]), ("meta", namedtuple)])
+
+        Raises:
+            SmartcarException: If an error occurs during the API call.
+
+        See Also:
+            Smartcar API Documentation for Vehicle Service History:
+            https://smartcar.com/docs/api#get-vehicle-service-history
+        """
+        path = "service/history"
+        url = self._format_url(path)
+        headers = self._get_headers()
+        params = {}
+        if start_date:
+            params["startDate"] = start_date
+        if end_date:
+            params["endDate"] = end_date
+
+        response = helpers.requester("GET", url, headers=headers, params=params)
+        return v2.select_named_tuple(path, response)
+
+    def diagnostic_system_status(self) -> v2.DiagnosticSystemStatus:
+        """
+        GET Vehicle.diagnostic_system_status
+
+        Returns:
+            DiagnosticSystemStatus: NamedTuple("DiagnosticSystemStatus", [
+                ("systems", List[DiagnosticSystem]),
+                ("meta", namedtuple)
+            ])
+
+        Raises:
+            SmartcarException
+        """
+        path = "diagnostics/system_status"
+        url = self._format_url(path)
+        headers = self._get_headers()
+        response = helpers.requester("GET", url, headers=headers)
+        return v2.select_named_tuple(path, response)
+
+    def diagnostic_trouble_codes(self) -> v2.DiagnosticTroubleCodes:
+        """
+        GET Vehicle.diagnostic_trouble_codes
+
+        Returns:
+            DiagnosticTroubleCodes: NamedTuple("DiagnosticTroubleCodes", [
+                ("active_codes", List[DiagnosticTroubleCode]),
+                ("meta", namedtuple)
+            ])
+
+        Raises:
+            SmartcarException
+        """
+        path = "diagnostics/dtcs"
+        url = self._format_url(path)
+        headers = self._get_headers()
+        response = helpers.requester("GET", url, headers=headers)
+        return v2.select_named_tuple(path, response)
+
+    def location(self) -> v2.Location:
         """
         GET Vehicle.location
 
@@ -198,7 +336,7 @@ class Vehicle(object):
         url = self._format_url(path)
         headers = self._get_headers()
         response = helpers.requester("GET", url, headers=headers)
-        return types.select_named_tuple(path, response)
+        return v2.select_named_tuple(path, response)
 
     def permissions(self, paging: dict = None):
         """
@@ -229,9 +367,9 @@ class Vehicle(object):
                 "GET", url, headers=headers, params={"limit": limit, "offset": offset}
             )
 
-        return types.select_named_tuple(path, response)
+        return v2.select_named_tuple(path, response)
 
-    def attributes(self) -> types.Attributes:
+    def attributes(self) -> v2.Attributes:
         """
         GET Vehicle.attributes
 
@@ -246,13 +384,53 @@ class Vehicle(object):
         url = self._format_url(path)
         headers = self._get_headers()
         response = helpers.requester("GET", url, headers=headers)
-        return types.select_named_tuple(path, response)
+        return v2.select_named_tuple(path, response)
+
+    def get_charge_limit(self) -> v2.ChargeLimit:
+        """
+        GET Vehicle.get_charge_limit
+
+        Returns:
+            ChargeLimit = NamedTuple("ChargeLimit", [("limit", float), ("meta", namedtuple)])
+
+        Raises:
+            SmartcarException
+        """
+        path = "charge/limit"
+        url = self._format_url(path)
+        headers = self._get_headers()
+        response = helpers.requester("GET", url, headers=headers)
+        return v2.select_named_tuple(path, response)
+
+    def lock_status(self) -> v2.LockStatus:
+        """
+            GET Vehicle.lock_status
+
+            Returns:
+                LockStatus = NamedTuple("LockStatus", [
+                    ("is_locked", bool)
+                    ("doors", List[Door]),
+                    ("windows", List[Window]),
+                    ("sunroof", List[Sunroof]),
+                    ("storage", List[Storage]),
+                    ("charging_port", List[ChargingPort]),
+                    ("meta", namedtuple)
+                ]
+        )
+            Raises:
+                SmartcarException
+        """
+        path = "security"
+        url = self._format_url(path)
+        headers = self._get_headers()
+        response = helpers.requester("GET", url, headers=headers)
+        return v2.select_named_tuple(path, response)
 
     # ===========================================
     # Action (POST) Requests
     # ===========================================
 
-    def lock(self) -> types.Status:
+    def lock(self) -> v2.Status:
         """
         POST Vehicle.lock
 
@@ -267,9 +445,9 @@ class Vehicle(object):
         response = helpers.requester(
             "POST", url, headers=headers, json={"action": "LOCK"}
         )
-        return types.select_named_tuple("lock", response)
+        return v2.select_named_tuple("lock", response)
 
-    def unlock(self) -> types.Status:
+    def unlock(self) -> v2.Status:
         """
         POST Vehicle.unlock
 
@@ -284,9 +462,9 @@ class Vehicle(object):
         response = helpers.requester(
             "POST", url, headers=headers, json={"action": "UNLOCK"}
         )
-        return types.select_named_tuple("unlock", response)
+        return v2.select_named_tuple("unlock", response)
 
-    def start_charge(self) -> types.Status:
+    def start_charge(self) -> v2.Status:
         """
         POST Vehicle.start_charge
 
@@ -301,9 +479,9 @@ class Vehicle(object):
         response = helpers.requester(
             "POST", url, headers=headers, json={"action": "START"}
         )
-        return types.select_named_tuple("start_charge", response)
+        return v2.select_named_tuple("start_charge", response)
 
-    def stop_charge(self) -> types.Status:
+    def stop_charge(self) -> v2.Status:
         """
         POST Vehicle.stop_charge
 
@@ -318,7 +496,68 @@ class Vehicle(object):
         response = helpers.requester(
             "POST", url, headers=headers, json={"action": "STOP"}
         )
-        return types.select_named_tuple("stop_charge", response)
+        return v2.select_named_tuple("stop_charge", response)
+
+    def set_charge_limit(self, limit) -> v2.Status:
+        """
+        POST Vehicle.set_charge_limit
+
+        Returns:
+            Action: NamedTuple("Action", [("status", str), ("message", str), ("meta", rs.namedtuple)])
+
+        Raises:
+            SmartcarException
+        """
+        url = self._format_url("charge/limit")
+        headers = self._get_headers(need_unit_system=False)
+        response = helpers.requester(
+            "POST", url, headers=headers, json={"limit": limit}
+        )
+        return v2.select_named_tuple("set_charge_limit", response)
+
+    def send_destination(self, latitude, longitude) -> v2.Action:
+        """
+        POST Vehicle.send_destination
+
+        Returns:
+            Action: NamedTuple("Action", [("status", str), ("message", str), ("meta", rs.namedtuple)])
+
+        Raises:
+            SmartcarException
+        """
+        url = self._format_url("navigation/destination")
+        headers = self._get_headers(need_unit_system=False)
+        response = helpers.requester(
+            "POST",
+            url,
+            headers=headers,
+            json={"latitude": latitude, "longitude": longitude},
+        )
+        return v2.select_named_tuple("send_destination", response)
+
+    @staticmethod
+    def _batch_path_response(
+        path: str, path_response: dict, top_response: dict
+    ) -> Callable[[], namedtuple]:
+        if path_response.get("code") == 200:
+            # attach top-level sc-request-id to res_dict
+            path_response["headers"]["sc-request-id"] = top_response.headers.get(
+                "sc-request-id"
+            )
+            # use lambda default args to avoid issues with closures
+            return lambda p=path, r=path_response: v2.select_named_tuple(p, r)
+
+        # if individual response is erroneous, attach a lambda that returns a SmartcarException
+        def _attribute_raise_exception(smartcar_exception):
+            raise smartcar_exception
+
+        path_status_code = path_response.get("code")
+        path_headers = path_response.get("headers", {})
+        path_body = json.dumps(path_response.get("body"))
+        sc_exception = sce.exception_factory(
+            path_status_code, path_headers, path_body, False
+        )
+        return lambda e=sc_exception: _attribute_raise_exception(e)
 
     def batch(self, paths: List[str]) -> namedtuple:
         """
@@ -355,44 +594,25 @@ class Vehicle(object):
         # success of the request.
         batch_dict = dict()
         path_responses = response.json()["responses"]
-        for res_dict in path_responses:
+        for path_response in path_responses:
             path, attribute = helpers.format_path_and_attribute_for_batch(
-                res_dict["path"]
+                path_response["path"]
+            )
+            batch_dict[attribute] = Vehicle._batch_path_response(
+                path, path_response, response
             )
 
-            if res_dict.get("code") == 200:
-                # attach top-level sc-request-id to res_dict
-                res_dict["headers"]["sc-request-id"] = response.headers.get(
-                    "sc-request-id"
-                )
-                # use lambda default args to avoid issues with closures
-                batch_dict[
-                    attribute
-                ] = lambda p=path, r=res_dict: types.select_named_tuple(p, r)
-            else:
-                # if individual response is erroneous, attach a lambda that returns a SmartcarException
-                def _attribute_raise_exception(smartcar_exception):
-                    raise smartcar_exception
-
-                code = res_dict.get("code")
-                headers = response.headers
-                body = json.dumps(res_dict.get("body"))
-                sc_exception = sce.exception_factory(code, headers, body)
-                batch_dict[
-                    attribute
-                ] = lambda e=sc_exception: _attribute_raise_exception(e)
-
         # STEP 3 - Attach Meta to batch_dict
-        batch_dict["meta"] = types.build_meta(response.headers)
+        batch_dict["meta"] = v2.build_meta(response.headers)
 
         # STEP 4 - Transform batch_dict into a namedtuple
-        return types.generate_named_tuple(batch_dict, "batch")
+        return v2.generate_named_tuple(batch_dict, "batch")
 
     # ===========================================
     # DELETE requests
     # ===========================================
 
-    def disconnect(self) -> types.Status:
+    def disconnect(self) -> v2.Status:
         """
         Disconnect this vehicle from the connected application.
 
@@ -409,13 +629,13 @@ class Vehicle(object):
         url = self._format_url("application")
         headers = self._get_headers(need_unit_system=False)
         response = helpers.requester("DELETE", url, headers=headers)
-        return types.select_named_tuple("disconnect", response)
+        return v2.select_named_tuple("disconnect", response)
 
     # ===========================================
     # Webhook requests
     # ===========================================
 
-    def subscribe(self, webhook_id: str) -> types.Subscribe:
+    def subscribe(self, webhook_id: str) -> v2.Subscribe:
         """
         Subscribe a vehicle to a webhook
 
@@ -428,9 +648,9 @@ class Vehicle(object):
         url = self._format_url(f"webhooks/{webhook_id}")
         headers = self._get_headers(need_unit_system=False)
         response = helpers.requester("POST", url, headers=headers)
-        return types.select_named_tuple("subscribe", response)
+        return v2.select_named_tuple("subscribe", response)
 
-    def unsubscribe(self, amt: str, webhook_id: str) -> types.Status:
+    def unsubscribe(self, amt: str, webhook_id: str) -> v2.Status:
         """
         Subscribe a vehicle to a webhook
 
@@ -446,7 +666,7 @@ class Vehicle(object):
         # Note: Authorization header is different, compared to the other methods
         headers = {"Authorization": f"Bearer {amt}"}
         response = helpers.requester("DELETE", url, headers=headers)
-        return types.select_named_tuple("unsubscribe", response)
+        return v2.select_named_tuple("unsubscribe", response)
 
     # ===========================================
     # General Purpose Request Method
@@ -454,7 +674,7 @@ class Vehicle(object):
 
     def request(
         self, method: str, path: str, body: dict = {}, headers: dict = {}
-    ) -> types.Response:
+    ) -> v2.Response:
         """
         Utility method to make a request to a Smartcar endpoint - can be used
         to make requests to brand specific endpoints.
@@ -483,7 +703,7 @@ class Vehicle(object):
 
         response = helpers.requester(method, url, headers=headers, json=body)
 
-        return types.select_named_tuple("request", response)
+        return v2.select_named_tuple("request", response)
 
     # ===========================================
     # Utility
@@ -517,12 +737,14 @@ class Vehicle(object):
             return f"?flags={helpers.format_flag_query(self._flags)}"
         return ""
 
-    def _format_url(self, path: str) -> str:
+    def _format_url(self, path: str, **kwargs) -> str:
         """
         Returns (str): Base url with current API version.
         User can change api_version attribute at will.
         """
-        return f"{config.API_URL}/v{self._api_version}/vehicles/{self.vehicle_id}/{path}{self._format_query_params()}"
+        origin = kwargs.get("origin") or config.API_ORIGIN
+        api_version = kwargs.get("api_version") or self._api_version
+        return f"{origin}/v{api_version}/vehicles/{self.vehicle_id}/{path}{self._format_query_params()}"
 
     def _get_headers(self, need_unit_system: bool = True) -> dict:
         """
